package cn.com.qmth.examcloud.core.basic.api.provider;

import cn.com.qmth.examcloud.commons.exception.StatusException;
import cn.com.qmth.examcloud.commons.util.ByteUtil;
import cn.com.qmth.examcloud.commons.util.SHA256;
import cn.com.qmth.examcloud.core.basic.api.StudentCloudService;
import cn.com.qmth.examcloud.core.basic.api.bean.StudentBean;
import cn.com.qmth.examcloud.core.basic.api.request.*;
import cn.com.qmth.examcloud.core.basic.api.response.*;
import cn.com.qmth.examcloud.core.basic.dao.OrgRepo;
import cn.com.qmth.examcloud.core.basic.dao.StudentRepo;
import cn.com.qmth.examcloud.core.basic.dao.UserRepo;
import cn.com.qmth.examcloud.core.basic.dao.entity.OrgEntity;
import cn.com.qmth.examcloud.core.basic.dao.entity.StudentEntity;
import cn.com.qmth.examcloud.core.basic.service.StudentService;
import cn.com.qmth.examcloud.core.basic.service.bean.StudentInfo;
import cn.com.qmth.examcloud.core.basic.service.cache.StudentCache;
import cn.com.qmth.examcloud.support.filestorage.FileStorageUtil;
import cn.com.qmth.examcloud.web.helpers.GlobalHelper;
import cn.com.qmth.examcloud.web.support.ControllerSupport;
import com.google.common.collect.Lists;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.jpa.domain.Specification;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.persistence.criteria.Predicate;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * {@link StatusException} 状态码范围：056XXX<br>
 *
 * @author WANGWEI
 * @date 2018年8月14日
 * @Copyright (c) 2018-? http://qmth.com.cn All Rights Reserved.
 */
@RestController
@Api(tags = "RPC-学生相关接口")
@RequestMapping("${$rmp.cloud.basic}" + "student")
public class StudentCloudServiceProvider extends ControllerSupport implements StudentCloudService {

    private static final long serialVersionUID = -571041158847398070L;

    @Autowired
    OrgRepo orgRepo;

    @Autowired
    UserRepo userRepo;

    @Autowired
    StudentService studentService;

    @Autowired
    StudentRepo studentRepo;

    @Autowired
    StudentCache studentCache;

    @ApiOperation(value = "保存学生")
    @PostMapping("saveStudent")
    @Transactional
    @Override
    public SaveStudentResp saveStudent(@RequestBody SaveStudentReq req) {

        trim(req);

        StudentInfo studentInfo = new StudentInfo();
        studentInfo.setEnable(req.getEnable());
        studentInfo.setName(req.getName());
        studentInfo.setIdentityNumber(req.getIdentityNumber());
        studentInfo.setStudentCodeList(req.getStudentCodeList());
        studentInfo.setRootOrgId(req.getRootOrgId());

        studentInfo.setOrgId(req.getOrgId());
        studentInfo.setOrgCode(req.getOrgCode());
        studentInfo.setOrgName(req.getOrgName());

        studentInfo.setPhoneNumber(req.getPhoneNumber());
        studentInfo.setPhotoPath(req.getPhotoPath());
        studentInfo.setRemark(req.getRemark());

        StudentEntity student = studentService.saveStudent(studentInfo);

        SaveStudentResp resp = new SaveStudentResp();
        resp.setStudentId(student.getId());
        resp.setRootOrgId(student.getRootOrgId());

        if (null != student.getOrgId()) {
            OrgEntity org = GlobalHelper.getEntity(orgRepo, student.getOrgId(), OrgEntity.class);
            resp.setOrgId(org.getId());
            resp.setOrgName(org.getName());
        }

        studentCache.remove(student.getId());

        return resp;
    }

    @ApiOperation(value = "查询学生")
    @PostMapping("getStudent")
    @Override
    public GetStudentResp getStudent(@RequestBody GetStudentReq req) {
        Long studentId = req.getStudentId();
        Long rootOrgId = req.getRootOrgId();
        String identityNumber = req.getIdentityNumber();
        String studentCode = req.getStudentCode();
        String securityPhone = req.getSecurityPhone();

        StudentInfo student = studentService.getStudentInfo(rootOrgId, studentId, identityNumber,
                studentCode, securityPhone);

        StudentBean studentBean = new StudentBean();
        studentBean.setId(student.getId());
        studentBean.setName(student.getName());
        studentBean.setIdentityNumber(student.getIdentityNumber());
        studentBean.setStudentCodeList(student.getStudentCodeList());
        studentBean.setRootOrgId(student.getRootOrgId());
        if (null != student.getOrgId()) {
            OrgEntity org = GlobalHelper.getEntity(orgRepo, student.getOrgId(), OrgEntity.class);
            studentBean.setOrgId(org.getId());
            studentBean.setOrgCode(org.getCode());
            studentBean.setOrgName(org.getName());
        }
        studentBean.setOrgId(student.getOrgId());
        studentBean.setOrgCode(student.getOrgCode());
        studentBean.setOrgName(student.getOrgName());
        studentBean.setPhoneNumber(student.getPhoneNumber());
        studentBean.setPhotoPath(student.getPhotoPath());
        studentBean.setRemark(student.getRemark());
        studentBean.setSecurityPhone(student.getSecurityPhone());

        GetStudentResp resp = new GetStudentResp();
        resp.setStudentInfo(studentBean);
        return resp;
    }

    @ApiOperation(value = "解绑学号")
    @PostMapping("unbindStudentCode")
    @Transactional
    @Override
    public UnbindStudentCodeResp unbindStudentCode(@RequestBody UnbindStudentCodeReq req) {
        Long rootOrgId = req.getRootOrgId();
        String studentCode = req.getStudentCode();
        String identityNumber = req.getIdentityNumber();

        if (null == rootOrgId) {
            throw new StatusException("160001", "rootOrgId is null");
        }
        if (StringUtils.isBlank(studentCode)) {
            throw new StatusException("160002", "studentCode is blank");
        }

        List<Long> studentIdList = studentService.unbindStudentCode(rootOrgId, studentCode,
                identityNumber);

        UnbindStudentCodeResp resp = new UnbindStudentCodeResp();
        resp.setStudentIdList(studentIdList);

        for (Long cur : studentIdList) {
            studentCache.remove(cur);
        }

        return resp;
    }

    @ApiOperation(value = "修改密码")
    @PostMapping("updatePassword")
    @Transactional
    @Override
    public UpdatePasswordResp updatePassword(@RequestBody UpdatePasswordReq req) {
        Long rootOrgId = req.getRootOrgId();
        String identityNumber = req.getIdentityNumber();
        Long studentId = req.getStudentId();
        String password = req.getPassword();

        StudentEntity entity = null;
        if (null != studentId) {
            entity = GlobalHelper.getEntity(studentRepo, studentId, StudentEntity.class);
        } else if (StringUtils.isNotBlank(identityNumber)) {
            if (null == rootOrgId) {
                throw new StatusException("056001", "rootOrgId is needed");
            }
            entity = studentRepo.findByIdentityNumberAndRootOrgId(identityNumber, rootOrgId);
        } else {
            throw new StatusException("056002", "params is missing");
        }

        if (null == entity) {
            throw new StatusException("056003", "no student");
        }

        byte[] bytes = SHA256.encode(entity.getIdentityNumber() + password);
        String encodePassword = ByteUtil.toHexAscii(bytes);
        entity.setPassword(encodePassword);
        studentRepo.save(entity);

        UpdatePasswordResp resp = new UpdatePasswordResp();
        return resp;
    }

    @ApiOperation(value = "更新学生状态")
    @PostMapping("updateStudentStatus")
    @Transactional
    @Override
    public UpdateStudentStatusResp updateStudentStatus(@RequestBody UpdateStudentStatusReq req) {
        Long rootOrgId = req.getRootOrgId();
        String identityNumber = req.getIdentityNumber();
        Long studentId = req.getStudentId();
        Boolean enable = req.getEnable();

        StudentEntity entity = null;
        if (null != studentId) {
            entity = GlobalHelper.getEntity(studentRepo, studentId, StudentEntity.class);
        } else if (StringUtils.isNotBlank(identityNumber)) {
            if (null == rootOrgId) {
                throw new StatusException("056004", "rootOrgId is needed");
            }
            entity = studentRepo.findByIdentityNumberAndRootOrgId(identityNumber, rootOrgId);
        } else {
            throw new StatusException("056005", "params is missing");
        }

        if (null == entity) {
            throw new StatusException("056006", "no student");
        }

        entity.setEnable(enable);
        StudentEntity saved = studentRepo.save(entity);

        studentCache.remove(saved.getId());

        UpdateStudentStatusResp resp = new UpdateStudentStatusResp();
        return resp;
    }

    @ApiOperation(value = "统计学生")
    @PostMapping("countStudent")
    @Override
    public CountStudentResp countStudent(@RequestBody CountStudentReq req) {
        Long rootOrgId = req.getRootOrgId();

        Specification<StudentEntity> specification = (root, query, cb) -> {
            List<Predicate> predicates = new ArrayList<>();
            predicates.add(cb.equal(root.get("rootOrgId"), rootOrgId));

            return cb.and(predicates.toArray(new Predicate[predicates.size()]));
        };

        long count = studentRepo.count(specification);

        CountStudentResp resp = new CountStudentResp();
        resp.setCount(count);
        return resp;
    }

    @ApiOperation(value = "根据ID获取学生集合")
    @PostMapping("getStudentListByIds")
    @Override
    public GetStudentListByIdsResp getStudentListByIds(@RequestBody GetStudentListByIdsReq req) {
        Long rootOrgId = req.getRootOrgId();
        List<Long> studentIdList = req.getStudentIdList();
        if (CollectionUtils.isEmpty(studentIdList)) {
            throw new StatusException("500", "studentIdList is empty");
        }
        if (1000 < studentIdList.size()) {
            throw new StatusException("500", "size of studentIdList is too large");
        }

        List<StudentEntity> studentEntityList = studentRepo.findByIdIn(studentIdList);

        List<StudentBean> list = Lists.newArrayList();
        for (StudentEntity student : studentEntityList) {
            if (!student.getRootOrgId().equals(rootOrgId)) {
                throw new StatusException("500", "params error");
            }
            list.add(toStudentBean(student));
        }

        GetStudentListByIdsResp resp = new GetStudentListByIdsResp();
        resp.setStudentBeanList(list);

        return resp;
    }

    @Override
    @PostMapping("getStudentMaps")
    @ApiOperation(value = "根据ID列表获取学生集合")
    public GetStudentMapsResp getStudentMaps(@RequestBody GetStudentMapsReq req) {
        Map<Long, StudentEntity> students = studentService.getStudentMapsByIds(req.getStudentIds());

        Map<Long, StudentBean> studentMaps = new HashMap<>();
        for (Map.Entry<Long, StudentEntity> entry : students.entrySet()) {
            StudentBean studentBean = toStudentBean(entry.getValue());

            if (StringUtils.isNotBlank(studentBean.getPhotoPath())) {
                studentBean.setPhotoPath(FileStorageUtil.realPath(FileStorageUtil.getIntactPath("student_base_photo", studentBean.getPhotoPath())));
            }

            studentMaps.put(entry.getKey(), studentBean);
        }

        GetStudentMapsResp resp = new GetStudentMapsResp();
        resp.setStudentMaps(studentMaps);
        return resp;
    }

    private StudentBean toStudentBean(StudentEntity entity) {
        StudentBean bean = new StudentBean();
        bean.setRootOrgId(entity.getRootOrgId());
        bean.setOrgId(entity.getOrgId());
        bean.setId(entity.getId());
        bean.setName(entity.getName());
        bean.setIdentityNumber(entity.getIdentityNumber());
        bean.setSecurityPhone(entity.getSecurityPhone());
        bean.setPhoneNumber(entity.getPhoneNumber());
        bean.setPhotoPath(entity.getPhotoPath());
        bean.setRemark(entity.getRemark());
        return bean;
    }

}
