package cn.com.qmth.examcloud.core.basic.starter;

import cn.com.qmth.examcloud.core.basic.dao.UniqueRuleHolder;
import cn.com.qmth.examcloud.core.basic.dao.enums.OrgProperty;
import cn.com.qmth.examcloud.core.basic.service.CryptoConfigService;
import cn.com.qmth.examcloud.core.basic.service.LoginRuleService;
import cn.com.qmth.examcloud.core.basic.service.cache.AppCache;
import cn.com.qmth.examcloud.support.filestorage.FileStorageUtil;
import cn.com.qmth.examcloud.web.jpa.DataIntegrityViolationTransverter;
import cn.com.qmth.examcloud.web.support.SpringContextHolder;
import org.apache.logging.log4j.ThreadContext;
import org.springframework.boot.SpringApplication;
import org.springframework.boot.autoconfigure.SpringBootApplication;
import org.springframework.boot.autoconfigure.domain.EntityScan;
import org.springframework.boot.autoconfigure.web.servlet.MultipartAutoConfiguration;
import org.springframework.cloud.client.discovery.EnableDiscoveryClient;
import org.springframework.cloud.netflix.eureka.EnableEurekaClient;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.ComponentScan;
import org.springframework.context.annotation.Configuration;
import org.springframework.data.jpa.repository.config.EnableJpaAuditing;
import org.springframework.data.jpa.repository.config.EnableJpaRepositories;
import org.springframework.transaction.annotation.EnableTransactionManagement;
import org.springframework.web.multipart.MultipartResolver;
import org.springframework.web.multipart.commons.CommonsMultipartResolver;

@Configuration
@EnableJpaAuditing
@EnableTransactionManagement
@EnableEurekaClient
@EnableDiscoveryClient
@ComponentScan(basePackages = {"cn.com.qmth"})
@EntityScan(basePackages = {"cn.com.qmth"})
@EnableJpaRepositories(basePackages = {"cn.com.qmth"})
@SpringBootApplication(exclude = {MultipartAutoConfiguration.class})
public class CoreBasicApp {

    static {
        if (System.getProperty("log.commonLevel") == null) {
            System.setProperty("log.commonLevel", "INFO");
        }
        System.setProperty("hibernate.dialect.storage_engine", "innodb");

        OrgProperty.init();
        DataIntegrityViolationTransverter.setUniqueRules(UniqueRuleHolder.getUniqueRuleList());
    }

    /**
     * main
     *
     * @param args
     * @throws Exception
     * @author WANGWEI
     */
    public static void main(String[] args) throws Exception {
        // AppBootstrap.run(CoreBasicApp.class, args);
        SpringApplication.run(CoreBasicApp.class, args);

        initCache();
        FileStorageUtil.initYunSite();
        FileStorageUtil.initYunClient();
    }

    /**
     * 初始化缓存
     *
     * @author WANGWEI
     */
    private static void initCache() {
        new Thread(new Runnable() {
            @Override
            public void run() {
                ThreadContext.put("TRACE_ID", Thread.currentThread().getName());

                SpringContextHolder.getBean(AppCache.class).loadAllFromResource();

                SpringContextHolder.getBean(LoginRuleService.class).refreshAllLoginRule();

                SpringContextHolder.getBean(CryptoConfigService.class).initCryptoConfigs();
            }
        }, "cache").start();
    }

    @Bean(name = "multipartResolver")
    public MultipartResolver multipartResolver() {
        CommonsMultipartResolver resolver = new CommonsMultipartResolver();
        resolver.setDefaultEncoding("UTF-8");
        resolver.setResolveLazily(true);
        resolver.setMaxInMemorySize(2);
        resolver.setMaxUploadSize(200 * 1024 * 1024);
        return resolver;
    }

}
